// T23estDisc.sce
// ESTIMATION OF DISCRETE MODEL
//    f(y(t)|u(t),y(t-1)) with y,u from {0,1}
// Experiments
// - change number of values of individual variables
// - increase the model order
// -------------------------------------------------------------------
exec("ScIntro.sce",-1)

// SIMULATION
nd=500;                       // number of steps
// parameters of simulation
thS= [.2 .8 
      .6 .4 
      .9 .1 
      .3 .7];
//thS=[1 0;1 0;0 1;0 1];        // alterntive parameters
ut=(rand(1,nd,'u')>.3)+1;     // control variable P(ut=1)=.3, P(ut=2)=.7
yt(1)=1;                      // initial condition for output

// time loop of simulation
for t=2:nd                    
  i=2*(ut(t)-1)+yt(t-1);       // row in the model parameter
  yt(t)=(rand(1,1,'u')>thS(i,1))+1;  // generation of the output
end

// ESTIMATION
V=zeros(4,2);                 // initial statistics
for t=2:nd
  i=2*(ut(t)-1)+yt(t-1);      // row of model matrix
  V(i,yt(t))=V(i,yt(t))+1;    // updt of statistics
  thp=V./(sum(V,2)*ones(1,2)); // pt estimates of parameters
  thE(:,t)=thp(:,1);
end

// Results
set(scf(1),'position',[600 60 600 500])
for i=1:4
  subplot(4,1,i)
  plot(thE(i,:))                                  // estimated
  plot((nd-199:nd),ones(1,200)*thS(i,1),':r','linewidth',2)     // true
  set(gca(),'data_bounds',[0 nd -.1 1.1])
  if i==1, 
    title('Evolution ot parameter estimates (left column, only)')
    legend('estimated','true',[350 1.2]);
  end
end

disp(thS,'Simulated parameter')
disp(thp,'Estimated parameter')

/*
s=1:20;
scf();
subplot(211),plot(s,ut(s),'.:'),title input
subplot(212),plot(s,yt(s),'.:'),title output
