// T32preCont_Adapt2.sce
// N-STEP PREDICTION WITH CONTINUOUS MODEL (WITh ESTIMATION)
// The model for simulation differs from that for estimation
// i.e. MODEL STRUCTURE MISMATCH is tackled.
// Experiments
// Change: - np = number of steps of prediction
//         - r  = noise variance (effect on estimation)
//         - th = model parametrs
//         - u  = input signal (effect on estimation)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1),mode(0)

nd=100;                            // number of data
np=3;                              // length of prediction (np>=1)
//  b0 a1 b1  a2 b2 a3 b3 k
th=[1 .4 -.3 -.5 .1 .6 .1 1]';     // regression coefficients
r=.2;                              // noise variance
u=sin(4*%pi*(1:nd)/nd)+rand(1,nd,'norm'); // input
y(1)=1; y(2)=-1; y(3)=0;           // prior data
Eth=rand(8,1,'n');                 // prior parametrs

nu=zeros(4,2);
yp=ones(1,nd);
nz=4;
for t=nz:(nd-np)                   // time loop (on-line tasks)
  // prediction
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) y(t-3) u(t-3) 1]';  // regression vector
  yy=ps'*Eth;                      // first prediction at t+1
  for j=1:np                       // loop of predictions for t+2,...,t+np
    tj=t+j;                        // future times for prediction
    ps=[u(tj); yy; ps(1:$-3); 1];  // reg.vecs with predicted outputs
    yy=ps'*Eth;                    // new prediction (partial)
  end
  yp(t+np)=yy;                     // final prediction for time t+np

  // simulation
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) y(t-3) u(t-3) 1]';  // regression vector for sim.
  y(t)=ps'*th+sqrt(r)*rand(1,1,'norm'); // output generation

  // estimation
  Ps=[y(t) u(t) y(t-1) u(t-1) y(t-2) u(t-2) y(t-3) u(t-3) 1]';  // reg.vect. for estim.
  if t==nz, V=1e-8*eye(length(Ps)); end // initial information matrix
  V=V+Ps*Ps';                           // update of statistics
  Vp=V(2:$,2:$);
  Vyp=V(2:$,1);
  Eth=inv(Vp+1e-8*eye(Vp))*Vyp;         // point estimates
  Et(:,t)=Eth(:,1);                     // stor est. parameters
end

// Results
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)

set(scf(1),'position',[100 100 1200 400]);
subplot(121),plot(Et')
set(gca(),"data_bounds",[0 nd+1 -1 2])
title('Evolution of estimated parameters')
legend('b0E','a1E','b1E','a2E','b2E','kE');
subplot(122)
s=(np+nz):(nd-np);
plot(s,y(s),'.:',s,yp(s),'rx')
set(gca(),"data_bounds",[1 nd -6 10])
legend('output','prediction');
title([string(np),'- step ahead prediction'])
