// T53ctrlDisc.sce
// CONTROL WITH DISCTRETE DYNAMIC MODEL
// Experiments
// Change: - criterion om
//         - uncertainty of the system
// -------------------------------------------------------------------
exec("ScIntro.sce",-1),getd()
function y=dsim(u,y1,th)
  // y=dsim(u,y1,th)    Simulation of a disctrete system
  // y   new output
  // u   input
  // y1  old input
  i=psi2row([u y1]);                           // row of the model matrix
  y=sum(rand(1,1,'unif')>cumsum(th(i,:)))+1;   // output
endfunction

// VARIABLES TO BE SET
nh=30;                        // length of control interval
y0=1;                         // initial condition for output

//y 1 2        u y1  =  criterion
om=[1 2     // 1 1
    2 3     // 1 2
    2 3     // 2 1
    3 4];   // 2 2  ... preference of lower indexes
  
//y  1  2      u y1  =  system model
th=[.9 .1   // 1 1
    .4 .6   // 1 2
    .8 .2   // 2 1
    .1 .9]; // 2 2   ... rather uncertain system

// computed variables and initializations
fs=zeros(1,2);                // remider after previous minimization

// CONTROL LAW COMPUTATION
for t=nh:-1:1
  fp=om+ones(4,1)*fs;         // penalty + reminder from last step
  // expectation
  f=sum((fp.*th),'c');        // expectation over y
  // minimization
  if f(1)<f(3),               // for y(t-1)=1
    us(t,1)=1; fs(1)=f(1);    // optimal control, minimum of criterion
  else 
    us(t,1)=2; fs(1)=f(3);    // optimal control, minimum of criterion
  end
  if f(2)<f(4),               // for y(t-1)=2
    us(t,2)=1; fs(2)=f(2);    // optimal control, minimum of criterion
  else 
    us(t,2)=2; fs(2)=f(4);    // optimal control, minimum of criterion
  end
end
J=fs(y0);                     // final value of criterion

// CONTROL APPLICATION 
y(1)=y0;
for t=1:nh
  u(t+1)=us(t,y(t));          // optimal control
  y(t+1)=dsim(u(t+1),y(t),th); // simulation
end

// RESULTS
plot(1:nh+1,y,'ro',1:nh+1,u,'g+')
set(gcf(),"position",[700 100 600 500])
legend('output','input')
set(gca(),'data_bounds',[.8 nh+.2, .8 2.2])
title('Optimal control with discrete model')

printf('\n Minimal value of the expectation of criterion: %g\n',J)
